open GMain
open Genlex

type var = Var of string;;

type typ = 
  | SINGLE of string
  | IMPL of typ * typ
  | PRODUIT of typ * typ
  | SOMME of typ * typ ;;

type terme = 
    V of var
  | App of terme * terme
  | Abs of var * terme
  | Pair of terme * terme
  | Fst of terme
  | Snd of terme
  | Injl of typ * typ * terme
  | Injr of typ * typ * terme
  | Case of typ * typ * var * var * terme * terme * terme;;

type assoc = ASSOC of var * typ ;;

type sequent = { env : assoc list ; lambda_terme : terme ; typage : typ} ;;

exception RegleNonApplicable ;;
exception Erreur_syntaxe ;;
exception Cas_impossible;;
exception Non_unifiable;;

(*------------       Analyseur lexicale et syntaxique       -------------*)

let keywords =
[ "lambda" ; "fst" ; "snd" ; "injl" ; "injr" ; "case" ; "of" ; "->" ; "." ;
  "(" ; ")" ; "," ; "|" ;"+";"*"];;

let analyseur_lexical2 l = Genlex.make_lexer keywords l ;;

(* analyseur syntaxique des types *)
let types t =
  try(
    let rec types_rec = parser
	[< 'Ident s >] -> SINGLE s 
      | [< 'Kwd "(" ; t = type_composite ; 'Kwd ")" >] -> t
    and type_composite = parser
      | [< arg = types_rec ; t = (autre_type arg) >] -> t
    and autre_type f = parser
      | [< 'Kwd "->" ; t = types_rec >] -> IMPL(f,t)
      | [< 'Kwd "+" ; t = types_rec >] -> SOMME(f,t)
      | [< 'Kwd "*" ; t = types_rec >] -> PRODUIT(f,t)
      | [<>] -> f
    in
    types_rec t
   ) with Stream.Error ("") -> raise Erreur_syntaxe;;
 
(* analyseur syntaxique de l'env *)
let env e =
  try (
    let rec env_rec = parser 
      | [< 'Kwd "(" ; 'Ident x ; 'Kwd "," ; 
	   a = types ; 'Kwd ")" ; s = suiv >] ->  (ASSOC(Var x, a))::s
      | [<>] -> []
	    
    and suiv = parser 
      | [< 'Kwd "," ; s = env_rec >] -> s
      | [<>] -> []
    in
    env_rec e
   ) with Stream.Error ("") -> raise Erreur_syntaxe;;


(* analyseur des termes *)
let termes l =
  try(
    let rec termes_rec =  parser
      | [< 'Ident c >] -> V (Var c)
      | [< 'Kwd "(" ; t = lambda ; 'Kwd ")" >] -> t 
	    
    and app = parser
      | [< t =termes_rec ; lam = (other_applications t) >] -> lam
	    
    and other_applications f = parser
      | [< arg = termes_rec ; stream >] ->
	  other_applications (App (f, arg)) stream
      | [< 'Kwd "," ; arg = termes_rec >]  -> Pair(f,arg) 
      | [<>] -> f
	    
    and lambda = parser
      | [< 'Kwd "lambda" ;'Ident c; 'Kwd ".";t1 = lambda >] ->Abs(Var c,t1)
      | [< 'Kwd "fst" ; t = termes_rec  >] -> Fst t
      | [< 'Kwd "snd" ; t = termes_rec >] -> Snd t
      | [< 'Kwd "injl" ; 'Kwd "(" ;  a = types ; 
	   'Kwd "," ; b = types ; 'Kwd ")" ; t = termes_rec >] -> Injl(a,b,t)
      | [< 'Kwd "injr" ; 'Kwd "(" ;  a = types ; 
	   'Kwd "," ; b = types ; 'Kwd ")" ; t = termes_rec >] -> Injr(a,b,t)
      | [< 'Kwd "case" ; 'Kwd "(" ;  a = types ; 
	   'Kwd "," ; b = types ; 'Kwd ")" ; t = termes_rec ; 'Kwd "of" ;
	   'Kwd "injl" ; 'Kwd "(" ;  c = types ; 'Kwd "," ; d = types ; 
	   'Kwd ")" ; 'Kwd "(" ; 'Ident v1 ; 'Kwd ")" ;
	   'Kwd "->" ; t1 = termes_rec ; 'Kwd "|" ; 'Kwd "injr" ; 
	   'Kwd "(" ;  e = types ; 'Kwd "," ; f = types ; 'Kwd ")" ; 
	   'Kwd "(" ; 'Ident v2 ; 'Kwd ")" ; 'Kwd "->" ; t2 = termes_rec >] ->
	     if ( (a= c) & ( a = e) & (b = d) & (b = f)) then  
	       Case(a,b,Var v1,Var v2,t,t1,t2) 
	     else raise Erreur_syntaxe
      | [< t = app  >] -> t
    in termes_rec l 
   ) with Stream.Error ("") -> raise Erreur_syntaxe;;


(*---------------------------Noyau------------------------------*)

let i = ref(0);;

(* genere une chaine de caractres du type "ai" ou i est un entier *)
let generer () =
   (i:=(!i)+1;
   ("a")^(string_of_int !i));;

(* retourne l'indice de couple dans l contenant f comme premier membre *)
let rec find_list l f = 
  ( 
    match l with
      [] -> -1 ;
    | t::q -> (
	match t with 
	  ASSOC(Var v1,t) -> 
	    (
	     match f with 
	       V(Var v2) -> 
		 if (v1 = v2) then 0 
		 else (1+ (find_list (List.tl l) f))
	     |  _ -> -1
	    )
       )
   );;


(* transforme un type en une chaine de caractres *)
let rec string_of t = 
  match t with
    SINGLE s -> s
  | SOMME(s1,s2) -> "("^(string_of s1)^"+"^ (string_of s2)^")"
  | PRODUIT(s1,s2) -> "("^(string_of s1)^"*"^ (string_of s2)^")"
  | IMPL(s1,s2) -> "("^(string_of s1)^"->"^ (string_of s2)^")";;

(* transforme une variable en une chaine de caractres *)
let string_of_var v = 
  match v with
    ( Var v1) -> v1
  | _ -> "";;

(* transforme un terme en une chaine de caractres *)
let rec trans_to_string t = 
  match t with 
    V(Var v) -> v
  | Abs(v,t1) -> "( lambda "^(string_of_var v)^"."^(trans_to_string t1)^")"
  | App(t1,t2) -> "( "^(trans_to_string t1)^" "^(trans_to_string t2)^" )"
  | Pair(t1,t2) -> "( "^(trans_to_string t1)^" , "^(trans_to_string t2)^" )"
  | Fst(t1) -> " fst( "^(trans_to_string t1)^" )"
  | Snd(t1) -> " snd( "^(trans_to_string t1)^" )"
  | Injl(a,b,t) -> 
      "injl"^"("^(string_of a)^","^(string_of b)^")"^
      "("^(trans_to_string t)^")"
  | Injr(a,b,t) -> 
      "injr"^"("^(string_of a)^","^(string_of b)^")"^
      "("^(trans_to_string t)^")"
  | Case(a,b,x,y,t,t1,t2) -> 
      "case"^"("^(string_of a)^" , "^(string_of b)^") "^
      (trans_to_string t)^" of injl("^(string_of a)^" , "^
      (string_of b)^")"^"("^(string_of_var x)^") ->"^
      (trans_to_string t1)^" | "^" injr("^(string_of a)^" , "^
      (string_of b)^")"^"("^(string_of_var y)^") ->"^(trans_to_string t2) ;;
      
(* transforme une association en une chaine de caractres *)
let affiche_assoc a = 
  match a with
  | ASSOC(v,t) -> "( "^(string_of_var v)^" , "^(string_of t)^" )"
  | _ -> "";;

(* transforme une liste d'associations en une chaine de caractres *)
let rec trans_to_string2 e = 
  match e with
  | t::q -> 
      if (q <> []) then ( affiche_assoc t )^","^(trans_to_string2 q)
      else ( affiche_assoc t )
  | [] -> "";;


(* ---------------------REGLES DE TYPAGE--------------------- *)

(* retourne le couple de types suite  la substitution de ct1 par ct2 *) 
let subst ct1 ct2 =
  let rec subst_rec t1 =
    match t1 with
    | SINGLE a ->(
	match (fst ct2) with 
	| SINGLE b -> if (a = b) then (snd ct2) else t1
	| _ -> t1
       )
    | IMPL (a,b) -> IMPL ((subst_rec a ),(subst_rec b ))
    | SOMME (a,b) -> SOMME((subst_rec a ),(subst_rec b ))
    | PRODUIT (a,b) -> PRODUIT((subst_rec a ),(subst_rec b ))
  in 
  let c = (subst_rec (snd ct1)) in 
  if ((fst ct1) <> c) then ((fst ct1),c) else ct1;;

(* teste les eventuels cycles dans un contrainte qui est  substituer *)
let test_occurence tg td =
  let rec test_rec t = 
    match t with
    | SINGLE a -> 
	 (
	  match tg with
	    SINGLE b -> (a = b) && (t <> td);    
	  | _ ->  (raise Cas_impossible)
	 )
    | IMPL(a,b)|SOMME(a,b)|PRODUIT(a,b) -> (test_rec a)||(test_rec b)
  in test_rec td ;;

(* teste les connecteurs dans un contrainte de type *) 
let test_connecte d g =
  let rec test_rec t1 t2=
    match t1 with 
      SINGLE a -> 
	if (test_occurence t1 t2) then false
	else true 
    |IMPL(a,b) ->
	(
	 match t2 with 
	 | IMPL(c,d)->((test_rec a c) && (test_rec b d))
	 | SINGLE s -> true
	 | _ -> false
	)
    | SOMME(a,b)->
	(
	 match t2 with 
	 | SOMME(c,d)-> ((test_rec a c) && (test_rec b d))
	 | SINGLE s -> true
	 | _ -> false
	)
    | PRODUIT(a,b)->
	(
	 match t2 with 
	 | PRODUIT(c,d)-> ((test_rec a c) && (test_rec b d))
	 | SINGLE s -> true
	 | _ -> false
	)
  in test_rec d g;;

(* fait appel  subst pour substituer t1 par t2 si possible *)	  
let unifie t1 t2 =
  let g = fst t2 in
  let d = snd t2 in
  match g with 
  | SINGLE a -> 
      if ((test_occurence g d) = false) then subst t1 t2 
      else raise Non_unifiable
  | _ -> if (test_connecte g d) then subst t1 t2 else raise Non_unifiable
;;


let unifie2 li = 
  let rec aux1 l1 l2 = 
    match l1 with 
    | []-> []
    | ct1::tl -> (aux2 l2 ct1)@(aux1 tl l2)
  and aux2 l2 c =
    match l2 with   
    | [] -> [c]
    | ct2::l ->(let a = (unifie c ct2) in if (c = a) then (aux2 l c) else [a])
  in
  aux1 li li ;;
  
(* retourne des contraintes lmentaires  partir des contraites compos *)
let rec unifie3 t1 t2 = 
  match t1 with 
  | SINGLE s -> 
      if (t1<> t2) then 
	(
	 match t2 with 
	   SINGLE s2 -> [(t2,t1);(t1,t2)] 
	 | _-> [(t1,t2)]
	)
      else []
  | IMPL(a1,b1) -> 
      (
       match t2 with 
       | IMPL(a2,b2) -> (unifie3 a1 a2)@(unifie3 b1 b2)
       | SINGLE s -> unifie3 t2 t1
       | _ -> raise Non_unifiable
      )
  | SOMME(a1,b1) -> 
      (
       match t2 with 
       | SOMME(a2,b2) -> (unifie3 a1 a2)@(unifie3 b1 b2)
       | SINGLE s -> unifie3 t2 t1
       | _ -> raise Non_unifiable
      )
  | PRODUIT(a1,b1) -> 
      (
       match t2 with 
       | PRODUIT(a2,b2) -> (unifie3 a1 a2)@(unifie3 b1 b2)
       | SINGLE s -> unifie3 t2 t1
       | _ -> raise Non_unifiable
      );;

(* fait appel  unifie3 sur les couples ayant le meme premier membre *)
(* en prenant les lments de l'argument deux  deux *)
let unifie4 li =
   let rec aux1 l1 l2 =   
   match l1 with
   | [] -> []
   | ct1::tl -> (aux2 l2 ct1)@(aux1 tl l2)
   and aux2 l2 c =
   match l2 with
   | [] -> [c]
   | ct2::l ->
       if (((fst c)=(fst ct2)) && ((snd c) <> (snd ct2))) then  
	 (
	  if (l <> []) then 
	    (unifie3 (snd c) (snd ct2))@(aux2 l c) 
	  else (unifie3 (snd c) (snd ct2))
	 )
       else (aux2 l c)
   in
   aux1 li li ;;

(* retourne  une liste de contraintes deduit de la liste en argument *)
(* en faisant appel  unifie3 *)
let rec unifie5 li =
  match li with
  | [] -> []
  | ct::tl -> 
      let c =(unifie3 (fst ct) (snd ct)) in 
      if ((List.length c)=1 ) then (c@(unifie5 tl)) 
      else ([ct]@c@(unifie5 tl));;

let unifie_inv t1 t2 = 
  unifie t2 t1;;

(* elimine les doublons dans la liste en argument *)
let rec elim_doubl li = 
  match li with 
    [] -> []
  | t::q ->  
      if ((List.mem t (q)) = true) then (elim_doubl q)
      else [t]@(elim_doubl q)
;;


(*-----------*)

let regle_var s cnt= 
  (
   let ind = (find_list s.env s.lambda_terme) in
   if (ind > -1 ) then (
     match (List.nth s.env ind) with
       ASSOC (v,t) -> 
	 (
	  let ct1  = (t,s.typage) in
	  if ((List.mem ct1 (!cnt)) = false) then 
	    cnt := (!cnt) @ [ct1] ;
	  cnt := elim_doubl (unifie5 (unifie2 (unifie4 (!cnt))));
  	  s.typage = t;
	  [];
	 )
    )
   else  raise RegleNonApplicable 
  );;

let regle_app s cnt = 
  (
   match s.lambda_terme with
     App(t1,t2) ->
       ( 
	 let a = generer() in
	 let b = generer() in
	 let ct =  (SINGLE a , IMPL (SINGLE b , s.typage) ) in
	 if ((List.mem ct (!cnt)) = false) then cnt := (!cnt) @ [ct];
	 cnt := elim_doubl (unifie5 (unifie2 (unifie4 (!cnt))));
	 [{ env = s.env ; lambda_terme = t1 ; typage = SINGLE a}
	    ;
	  { env = s.env ; lambda_terme = t2 ; typage = SINGLE b }];
	)
   | _-> raise RegleNonApplicable
  );;


let regle_abs s cnt =
  ( 
    match s.lambda_terme with
      Abs(v,t) ->
	(
	 let a = generer() in
	 let b = generer() in
	 let ct = (s.typage,IMPL(SINGLE a ,SINGLE b)) in
	 if ((List.mem ct (!cnt)) = false) then cnt := (!cnt) @ [ct];
	 cnt := elim_doubl (unifie5 (unifie2 (unifie4 (!cnt))));
	 [{ env = ASSOC(v,SINGLE a)::s.env ; 
	    lambda_terme = t ; 
	    typage = SINGLE b 
	  }];
	)
    | _-> raise RegleNonApplicable );;

let regle_pair s cnt=
  (
   match s.lambda_terme with
     Pair(t1,t2) ->
       (
	let a = generer() in
	let b = generer() in
	let ct = (s.typage,PRODUIT(SINGLE a, SINGLE b)) in
	if ((List.mem ct (!cnt)) = false) then cnt := (!cnt) @ [ct];
	cnt := elim_doubl (unifie5 (unifie2 (unifie4 (!cnt))));
	[{ env = s.env ; lambda_terme = t1 ; typage = SINGLE a}
	   ;
	 { env = s.env ; lambda_terme = t2 ; typage = SINGLE b
	 }];
       )
   | _-> raise RegleNonApplicable 
  );;

let regle_fst s cnt = 
  (
   match s.lambda_terme with
     Fst(t) ->
       (
	let a = generer() in
	let b = generer() in
	let ct = (SINGLE a,PRODUIT(s.typage,SINGLE b)) in
	if ((List.mem ct (!cnt)) = false) then cnt := (!cnt) @ [ct];
	cnt := elim_doubl (unifie5 (unifie2 (unifie4 (!cnt))));
	[{ env = s.env ; lambda_terme = t ;typage = SINGLE a}];
       )
   | _-> raise RegleNonApplicable
  );;

let regle_snd s cnt = 
  (
   match s.lambda_terme with
     Snd(t) ->
       (
	let a = generer() in
	let b = generer() in
	let ct = (SINGLE a,PRODUIT(SINGLE b,s.typage)) in
	if ((List.mem ct (!cnt)) = false) then cnt := (!cnt) @ [ct];
	cnt := elim_doubl (unifie5 (unifie2 (unifie4 (!cnt))));
	[{ env = s.env ; lambda_terme = t ;typage = SINGLE a}];
       )
   | _-> raise RegleNonApplicable
  );;

let regle_injl s cnt=
  (
   match s.lambda_terme with
     Injl(a,b,t) -> 
       (
	let ct = (s.typage,SOMME(a,b)) in
	if ((List.mem ct (!cnt)) = false) then cnt := (!cnt) @ [ct];
	cnt := elim_doubl (unifie5 (unifie2 (unifie4 (!cnt))));
	s.typage =SOMME(a,b);
	[{ env = s.env ; lambda_terme =t ; typage = a}]
       )
   | _-> raise RegleNonApplicable
  );;

let regle_injr s cnt=
  (
   match s.lambda_terme with
     Injr(a,b,t) -> 
       (
	let ct = (s.typage,SOMME(a,b)) in
	if ((List.mem ct (!cnt)) = false) then cnt := (!cnt) @ [ct];
	cnt := elim_doubl (unifie5 (unifie2 (unifie4 (!cnt))));
	s.typage =SOMME(a,b);
	[{ env = s.env ; lambda_terme =t ; typage = b}]
       )
   | _-> raise RegleNonApplicable
  );;
     
let regle_case s cnt= 
  (
   match s.lambda_terme with
     Case(a,b,v1,v2,t,t1,t2) -> 
       (
	let ce = generer() in
	let ct = (s.typage,SINGLE ce) in
	if ((List.mem ct (!cnt)) = false) then cnt := (!cnt) @ [ct];
	cnt := elim_doubl (unifie5 (unifie2 (unifie4 (!cnt))));
	s.typage = SINGLE ce;
	[{ env = s.env ; lambda_terme = t ; typage = SOMME(a,b) }
	   ;
	 { env =  ASSOC(v1,a)::s.env ; lambda_terme = t1 ; typage = SINGLE ce}
	   ;
	 { env =  ASSOC(v2,b)::s.env ; lambda_terme = t2 ; typage = SINGLE ce
	 }]
       )
   |_-> raise RegleNonApplicable 
  );;

(* ------------------ INTERFACE ------------------- *)

(* supprime les lments contenus dans une "box" *)
let rec vider b = 
  let l= b#children in
  for i=0 to ((List.length l)-1) do (
    b#remove (List.nth l i);
   )done
;;


(* affiche les lments de c dans b1 *)
let aff_cnt c b1 = 
  vider b1;
  let rec aff_cnt_rec c1 b2 =
    match c1 with
    |[] ->  
	GMisc.label ~text:"-------------------" ~packing:b1#add  
	  ~line_wrap:true ();
    |q::tl -> 
	(
	 let t1 = ((string_of (fst q))^" = "^(string_of (snd q))) in
	 (GMisc.label ~text:t1 ~packing:b2#add  
	    ~line_wrap:true ())#set_use_underline true ;
	 GMisc.label ~text:"-------------------" ~packing:b2#add  
	   ~line_wrap:true ();
	 aff_cnt_rec tl b2;
	)
  in
  aff_cnt_rec c b1;;


let main () =
  
  let window = GWindow.dialog ~title:" Typage " ~width:900 ~height:750
      ~border_width:0 () in
  window#connect#destroy ~callback:Main.quit;
  
  let scrolled_window = GBin.scrolled_window ~border_width:10
      ~hpolicy:`AUTOMATIC ~vpolicy:`AUTOMATIC ~packing:window#vbox#add ()  in  
  
  let box = GPack.hbox ~packing:scrolled_window#add_with_viewport ()  in
  
  let box0 = GPack.vbox ~spacing:10 ~border_width:50
      ~packing:box#pack () in
  
  let b0 = GButton.button ~label:"Recommencer"
      ~packing:box0#pack () in
  
  GMisc.label ~packing:box0#add  ~line_wrap:true () ;
  
  let frame = GBin.frame ~label:"Les Regles de typage"
      ~packing:box0#pack () in
  
  let box_err = GPack.hbox ~packing:box0#pack () in
  
  let erreur = GMisc.label ~packing:((box_err)#pack ~from:`END) 
      ~line_wrap:true () in
  
  GMisc.label ~packing:box0#add  ~line_wrap:true () ;
  
  let button0 = GButton.radio_button ~label:"Choisissez une regle"
      ~active:true ~packing:box0#add () in
  
  let button1 = GButton.radio_button
      ~group:button0#group ~label:"Var" ~packing:box0#add () in                                            
  let button2 = GButton.radio_button 
      ~group:button0#group ~label:"Abs" ~packing:box0#add () in
  
  let button3 = GButton.radio_button
      ~group:button0#group ~label:"App" ~packing:box0#add () in
  
  let button4 = GButton.radio_button
      ~group:button0#group ~label:"Pair" ~packing:box0#add () in
  
  let button5 = GButton.radio_button
      ~group:button0#group ~label:"Fst" ~packing:box0#add () in
  
  let button6 = GButton.radio_button
      ~group:button0#group ~label:"Snd" ~packing:box0#add () in 
  
  let button7 = GButton.radio_button
      ~group:button0#group ~label:"Injl" ~packing:box0#add () in 
  
  let button8 = GButton.radio_button
      ~group:button0#group ~label:"Injr" ~packing:box0#add () in 
  
  let button9 = GButton.radio_button
      ~group:button0#group ~label:"Case" ~packing:box0#add () in 
  
  let box1 = GPack.vbox ~packing:box#pack () in
  
  let box2 = GPack.vbox ~spacing:10 ~border_width:50 ~packing:box#pack () in
  
  let box3 = GPack.vbox ~packing:box2#pack () in
  
  GMisc.label ~packing:box3#add ~line_wrap:true ();
  
  let box01 =GPack.hbox ~spacing:10 ~border_width:10
      ~packing:(box1#pack ~from:`END) () in
  
  let entry0 = GEdit.entry ~text:"  " ~max_length:500
      ~packing:box01#add () in
  
  GMisc.label ~text:" |- " ~packing:box01#add ~line_wrap:true ();
  
  let entry1 = GEdit.entry ~text:"  " ~max_length:1000 ~width:400
      ~packing:box01#add () in
  
  let affiche_type = GMisc.label ~packing:box01#pack  ~line_wrap:true () in
  
  let separator = GMisc.separator `HORIZONTAL 
      ~packing:(box1#pack ~from:`END)  () in
  
  let branche_active =ref [box1] in
  
  let branche_visitee = ref [box1] in
  
  let list_sequent =ref [] in
  
  let sequent_visite = ref [] in
  
  let dern_regle = ref [] in
  
  let tab_types = ref [] in

(* la fonction concernant le bouton de la rgle Var *) 
  
  button1#connect#toggled  ~callback:(fun _-> 
    if (button1#active) then 
      ( 
	try( 
	  if (entry1#editable) then ( 
	    let a0 = entry0#text in
	    let a1 = entry1#text in
	    entry0#set_editable false;
	    entry1#set_editable false; 
	    if (a1 <> "  ")then ( 
	      let f0 = Stream.of_string a0 in
	      let f = Stream.of_string a1 in 
	      let l0 = analyseur_lexical2 f0 in
	      let l = analyseur_lexical2 f in
	      let t = (termes l )in 
	      let e = env l0 in
	      let s1 = { env = e ; lambda_terme = t ; 
			 typage = SINGLE( generer() ) 
		       } in
	      let s = regle_var s1 tab_types in
	      tab_types := 
		elim_doubl(unifie5 (unifie2 (unifie4 (!tab_types))));
	      let c = List.fold_right unifie_inv 
		  (List.tl (!tab_types)) (List.hd (!tab_types)) in
	      tab_types := [c]@(List.tl (!tab_types));
	      aff_cnt (!tab_types) box3;
	      let box = List.hd (!branche_active) in
	      erreur#set_text "  ";
	      (GMisc.label  ~text:"(VAR)" ~xalign:0.0 ~yalign:0.0 
		 ~packing:((box)#pack ~from:`END) 
		 ~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	      list_sequent := [];
	      dern_regle := [] ;
	      erreur#set_text "TERMINE";
	      branche_active := List.tl (!branche_active);
	      affiche_type#set_text (string_of s1.typage);
	      for i=1 to 1000000 do()done ;
	      button0#set_active true
	     )else (
	      entry0#set_editable true;
	      entry1#set_editable true;
	      erreur#set_text "entrez un lambda terme";
	      button0#set_active true;
	     )
	   )
	  else 
	    (
	     let s1 = List.hd !list_sequent in
	     let s = regle_var s1 tab_types in
	     let box = List.hd (!branche_active) in
	     erreur#set_text "  ";
	     (GMisc.label  ~text:"(VAR)" ~xalign:0.0 ~yalign:0.0 
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	    (GMisc.label  ~packing:((box)#pack ~from:`END) 
	       ~line_wrap:true ())#set_use_underline true ;
	    (GMisc.label  ~packing:((box)#pack ~from:`END) 
	       ~line_wrap:true ())#set_use_underline true ;
	     sequent_visite := [List.hd (!list_sequent)]@(!sequent_visite);
	     list_sequent := List.tl (!list_sequent);
	     dern_regle := ["var"]@(!dern_regle) ;
	     if ((!list_sequent) = [])then (
	       erreur#set_text "TERMINE" ; 
	       tab_types := 
		 elim_doubl (unifie5 (unifie2 (unifie4 (!tab_types))));
	       let a = List.assoc (SINGLE "a1") (!tab_types) in
	       let c = List.fold_right unifie_inv 
		   (!tab_types) (SINGLE "a1",a) in
	       tab_types := [c]@(List.tl (!tab_types));
	       dern_regle :=[];
	      );
	     aff_cnt (!tab_types) box3;
             if (box <> (List.hd (!branche_visitee))) then  
	       branche_visitee := [box]@(!branche_visitee);
	     branche_active := List.tl (!branche_active);
	     for i=1 to 1000000 do()done ;
	     button0#set_active true
	    )
	 )with RegleNonApplicable -> (
	   erreur#set_text "RegleNonApplicable" ;
	   button0#set_active true)
	| Erreur_syntaxe ->  (
	    erreur#set_text "Erreur Syntaxique" ;
	    entry0#set_editable true;
	    entry1#set_editable true;
	    button0#set_active true
	   )  
       )
				     ) ;    
 
(* la fonction concernant le bouton de la rgle ABS *)  
  button2#connect#toggled  ~callback:(fun _->
    if (button2#active)then
      (
       try(
	 if (entry1#editable ) then (
	   let a0 = entry0#text in
	   let a1 = entry1#text in
	   entry0#set_editable false;
	   entry1#set_editable false;
           if (a1 <> "  ")then (
	     let f0 = Stream.of_string a0 in
	     let f = Stream.of_string a1 in 
	     let l0 = analyseur_lexical2 f0 in
	     let l = analyseur_lexical2 f in
	     let e = env l0 in
	     let t1 = (termes l )in 
	     let s1 = { env = e ; 
			lambda_terme = t1 ; 
			typage = SINGLE(generer()) 
		      } in
	     affiche_type#set_text (string_of s1.typage);
	     let s2 = List.hd ( regle_abs s1 tab_types) in
	     aff_cnt (!tab_types) box3;
	     let t = trans_to_string2 s2.env in
	     let txt =(t^" |- "^(trans_to_string s2.lambda_terme)
		       ^ " : "^(string_of(s2.typage))) in
	     let box = List.hd (!branche_active) in
	     erreur#set_text "  ";
	     (GMisc.label  ~text:"(ABS)" ~xalign:0.0 ~yalign:0.0  
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt 
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box)#pack ~from:`END) () in
	     list_sequent := [s2];
	     dern_regle := ["Abs"] ;
	     for i=1 to 1000000 do()done ;
             button0#set_active true
	    ) else (
	     entry0#set_editable true;
	     entry1#set_editable true;
	     erreur#set_text "entrez un lambda terme";
	     button0#set_active true;
	    )
	  )else(
	   let s1 = List.hd !list_sequent in
	   let s2 = List.hd ( regle_abs s1 tab_types) in 
	   let t = trans_to_string2 s2.env in
	   aff_cnt (!tab_types) box3;
	   sequent_visite := [List.hd (!list_sequent)]@(!sequent_visite);
	   list_sequent := List.tl (!list_sequent) ;
	   list_sequent := [(s2)]@(!list_sequent) ;
	   dern_regle := ["Abs"]@(!dern_regle);
	   let txt =(t^" |- "^(trans_to_string s2.lambda_terme)
		     ^ " : "^(string_of(s2.typage))) in
	   let box = List.hd (!branche_active) in
	   erreur#set_text "  ";
	   (GMisc.label  ~text:"(ABS)" ~xalign:0.0 ~yalign:0.0 
	      ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label ~text:txt 
	      ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   let separator = GMisc.separator `HORIZONTAL 
	       ~packing: ((box)#pack ~from:`END) () in
	   for i=1 to 1000000 do()done ;
	   if (box <> (List.hd (!branche_visitee))) then  
	     branche_visitee := [box]@(!branche_visitee);
	   button0#set_active true
	  ) 
	)with RegleNonApplicable -> (
	  erreur#set_text "RegleNonApplicable" ;
	  button0#set_active true
	 )
       | Erreur_syntaxe -> (
	   erreur#set_text "Erreur Syntaxique" ;
	   entry0#set_editable true;
	   entry1#set_editable true;
	   button0#set_active true
	  )
      )
				     ) ;

(* la fonction concernant le bouton de la rgle APP *) 
  button3#connect#toggled  ~callback:(fun _->
    if (button3#active)then
      (
       try(
	 if (entry1#editable ) then (
	   let a0 = entry0#text in
	   let a1 = entry1#text in
	   entry0#set_editable false;
	   entry1#set_editable false;
           if (a1 <> "  ")then (
	     let f0 = Stream.of_string a0 in
	     let f = Stream.of_string a1 in 
	     let l0 = analyseur_lexical2 f0 in
	     let l = analyseur_lexical2 f in
	     let t = (termes l )in 
	     let e = env l0 in
	     let s0  = { env = e ; 
			 lambda_terme = t ; 
			 typage = SINGLE(generer()) 
		       } in
	     affiche_type#set_text (string_of s0.typage);
	     let s = regle_app s0  tab_types in 
	     aff_cnt (!tab_types) box3;
	     let s1 = List.hd s in 
	     let s2 = List.nth s 1 in
	     let t1 = trans_to_string2 s1.env in
	     let t2 = trans_to_string2 s2.env in
	     let txt1 =(t1^" |- "^(trans_to_string s1.lambda_terme)
			^ " : "^(string_of(s1.typage))) in
	     let txt2 =(t2^" |- "^(trans_to_string s2.lambda_terme)
			^ " : "^(string_of(s2.typage))) in
	     let box = List.hd (!branche_active) in
	     erreur#set_text "  ";
	     let box02 =GPack.hbox ~homogeneous:true ~spacing:7
		 ~packing:((box)#pack ~from:`END) () in
	     let box03 = GPack.vbox ~packing:box02#pack () in
	     let box04 = GPack.vbox ~packing:box02#pack () in
	     (GMisc.label  ~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~text:"(APP)" ~xalign:0.0 ~yalign:0.0 
		~packing:((box03)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~packing:((box04)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt1 ~packing:((box03)#pack ~from:`END)  
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt2 ~packing:((box04)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box03)#pack ~from:`END) () in
             let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box04)#pack ~from:`END) () in
	     list_sequent := [s1;s2];
	     dern_regle := ["App"] ;
	     for i=1 to 1000000 do()done ;
	     branche_active := [box03]@[box04];
             button0#set_active true
	    ) else (
	     entry0#set_editable true;	
	     entry1#set_editable true;
	     erreur#set_text "entrez un lambda terme";
	     button0#set_active true;
	    )
	  )else(
	   let s0 = List.hd !list_sequent in
	   let s =  regle_app s0 tab_types in
	   aff_cnt (!tab_types) box3;
	   let s1 = List.hd s in
	   let s2 = List.nth s 1 in
	   let t1 = trans_to_string2 s1.env in
	   let t2 = trans_to_string2 s2.env in
	   let txt1 =(t1^" |- "^(trans_to_string s1.lambda_terme)
		      ^ " : "^(string_of(s1.typage))) in
	   let txt2 =(t2^" |- "^(trans_to_string s2.lambda_terme)
		      ^ " : "^(string_of(s2.typage)))  in
	   let box = List.hd (!branche_active) in
	   erreur#set_text "  ";
	   let box02 =GPack.hbox ~homogeneous:true ~spacing:7
	       ~packing:((box)#pack ~from:`END) () in
	   let box03 = GPack.vbox ~packing:box02#pack () in
	   let box04 = GPack.vbox ~packing:box02#pack () in
	   (GMisc.label  ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label  ~text:"(APP)" ~xalign:0.0 ~yalign:0.0 
	      ~packing:((box03)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label  ~packing:((box04)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label ~text:txt1 ~packing:((box03)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label ~text:txt2 ~packing:((box04)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label  ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   let separator = GMisc.separator `HORIZONTAL 
	       ~packing: ((box03)#pack ~from:`END) () in
	   let separator = GMisc.separator `HORIZONTAL 
	       ~packing: ((box04)#pack ~from:`END) () in
           sequent_visite := [List.hd (!list_sequent)]@(!sequent_visite);
	   list_sequent := List.tl (!list_sequent);
	   list_sequent := [s1;s2]@(!list_sequent);
	   dern_regle := ["App"]@(!dern_regle);
	   for i=1 to 1000000 do()done ;
	   if (box <> (List.hd (!branche_visitee))) then  
	     branche_visitee := [box]@(!branche_visitee);
	   branche_active := List.tl (!branche_active); 
	   branche_active := box03::box04::(!branche_active);
           button0#set_active true
	  ) 
	)with RegleNonApplicable -> (
	  erreur#set_text "RegleNonApplicable" ;
	  button0#set_active true
	 )
       | Erreur_syntaxe -> (
	   erreur#set_text "Erreur Syntaxique" ;
	   entry0#set_editable true;
	   entry1#set_editable true;
	   button0#set_active true
	  )
      )
				     ) ;

(* la fonction concernant le bouton de la rgle PAIR *) 
  button4#connect#toggled  ~callback:(fun _->
    if (button4#active)then
      (
       try(
	 if (entry1#editable ) then (
	   let a0 = entry0#text in
	   let a1 = entry1#text in
	   entry0#set_editable false;
	   entry1#set_editable false;
           if (a1 <> "  ")then (
	     let f0 = Stream.of_string a0 in
	     let f = Stream.of_string a1 in 
	     let l0 = analyseur_lexical2 f0 in
	     let l = analyseur_lexical2 f in
	     let t = (termes l )in 
	     let e = env l0 in
	     let s0  = { env = e ; 
			 lambda_terme = t ; 
			 typage = SINGLE(generer()) 
		       } in
	     affiche_type#set_text (string_of s0.typage);
	     let s = regle_pair s0  tab_types in 
	     aff_cnt (!tab_types) box3;
	     let s1 = List.hd s in 
	     let s2 = List.nth s 1 in
	     let t1 = trans_to_string2 s1.env in
	     let t2 = trans_to_string2 s2.env in
	     let txt1 =(t1^" |- "^(trans_to_string s1.lambda_terme)
			^ " : "^(string_of(s1.typage)))  in
	     let txt2 =(t2^" |- "^(trans_to_string s2.lambda_terme)
			^ " : "^(string_of(s2.typage)))  in
	     let box = List.hd (!branche_active) in
	     erreur#set_text "  ";
	     let box02 =GPack.hbox ~homogeneous:true ~spacing:7
		 ~packing:((box)#pack ~from:`END) () in
	     let box03 = GPack.vbox ~packing:box02#pack () in
	     let box04 = GPack.vbox ~packing:box02#pack () in
	     (GMisc.label  ~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~packing:((box03)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~packing:((box04)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt1 ~packing:((box03)#pack ~from:`END)  
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt2 ~packing:((box04)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box03)#pack ~from:`END) () in
             let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box04)#pack ~from:`END) () in
	     list_sequent := [s1;s2];
	     dern_regle := ["Pair"] ;
	     for i=1 to 1000000 do()done ;
	     branche_active := [box03]@[box04];
             button0#set_active true
	    ) else (
	     entry0#set_editable true;
	     entry1#set_editable true;
	     erreur#set_text "entrez un lambda terme";
	     button0#set_active true;
	    )
	  )else(
	   let s0 = List.hd !list_sequent in
	   let s =  regle_pair s0 tab_types in
	   aff_cnt (!tab_types) box3;
	   let s1 = List.hd s in
	   let s2 = List.nth s 1 in
	   let t1 = trans_to_string2 s1.env in
	   let t2 = trans_to_string2 s2.env in
	   let txt1 =(t1^" |- "^(trans_to_string s1.lambda_terme)
		      ^ " : "^(string_of(s1.typage)))  in
	   let txt2 =(t2^" |- "^(trans_to_string s2.lambda_terme)
		      ^ " : "^(string_of(s2.typage)))  in
	   let box = List.hd (!branche_active) in
	   erreur#set_text "  ";
	   let box02 =GPack.hbox ~homogeneous:true ~spacing:7
	       ~packing:((box)#pack ~from:`END) () in
	   let box03 = GPack.vbox ~packing:box02#pack () in
	   let box04 = GPack.vbox ~packing:box02#pack () in
	   (GMisc.label  ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label  ~packing:((box03)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label  ~packing:((box04)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label ~text:txt1 ~packing:((box03)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label ~text:txt2 ~packing:((box04)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label  ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   let separator = GMisc.separator `HORIZONTAL 
	       ~packing: ((box03)#pack ~from:`END) () in
	   let separator = GMisc.separator `HORIZONTAL 
	       ~packing: ((box04)#pack ~from:`END) () in
           sequent_visite := [List.hd (!list_sequent)]@(!sequent_visite);
	   list_sequent := List.tl (!list_sequent);
	   list_sequent := [s1;s2]@(!list_sequent);
	   dern_regle := ["Pair"]@(!dern_regle);
	   for i=1 to 1000000 do()done ;
	   if (box <> (List.hd (!branche_visitee))) then  
	     branche_visitee := [box]@(!branche_visitee);
	   branche_active := List.tl (!branche_active); 
	   branche_active := box03::box04::(!branche_active);
           button0#set_active true
	  ) 
	)with RegleNonApplicable -> (
	  erreur#set_text "RegleNonApplicable" ;
	  button0#set_active true
	 )
       | Erreur_syntaxe -> (
	   erreur#set_text "Erreur Syntaxique" ;
	   entry0#set_editable true;
	   entry1#set_editable true;
	   button0#set_active true
	  )
      )
				     ) ;
  
(* la fonction concernant le bouton de la rgle FST *) 
  button5#connect#toggled  ~callback:(fun _->
    if (button5#active)then
      (
       try(
	 if (entry1#editable ) then (
	   let a0 = entry0#text in
	   let a1 = entry1#text in
	   entry0#set_editable false;
	   entry1#set_editable false;
           if (a1 <> "  ")then (
	     let f0 = Stream.of_string a0 in
	     let f = Stream.of_string a1 in 
	     let l0 = analyseur_lexical2 f0 in
	     let l = analyseur_lexical2 f in
	     let t1 = (termes l )in
	     let e = env l0 in
	     let s1 = { env = e ; 
			lambda_terme = t1 ; 
			typage = SINGLE(generer()) 
		      } in
	     affiche_type#set_text (string_of s1.typage);
	     let s2 = List.hd ( regle_fst s1 tab_types) in
	     aff_cnt (!tab_types) box3;
	     let t = trans_to_string2 s2.env in
	     let txt =(t^" |- "^(trans_to_string s2.lambda_terme)
		       ^ " : "^(string_of(s2.typage)))  in
	     let box = List.hd (!branche_active) in
	     erreur#set_text "  ";
	     (GMisc.label  ~text:"(FST)" ~xalign:0.0 ~yalign:0.0 
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt 
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box)#pack ~from:`END) () in
	     list_sequent := [s2];
	     dern_regle := ["Fst"] ;
	     for i=1 to 1000000 do()done ;
             button0#set_active true
	    ) else (
	     entry0#set_editable true;
	     entry1#set_editable true;
	     erreur#set_text "entrez un lambda terme";
	     button0#set_active true;
	    )
	  )else(
	   let s1 = List.hd !list_sequent in
	   let s2 = List.hd ( regle_fst s1 tab_types) in 
	   aff_cnt (!tab_types) box3;
	   let t = trans_to_string2 s2.env in
	   sequent_visite := [List.hd (!list_sequent)]@(!sequent_visite);
	   list_sequent := List.tl (!list_sequent) ;
	   list_sequent := [(s2)]@(!list_sequent) ;
	   dern_regle := ["Fst"]@(!dern_regle);
	   let txt =(t^" |- "^(trans_to_string s2.lambda_terme)
		     ^ " : "^(string_of(s2.typage)))  in
	   let box = List.hd (!branche_active) in
	   erreur#set_text "  ";
	   (GMisc.label  ~text:"(FST)" ~xalign:0.0 ~yalign:0.0 
	      ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label ~text:txt 
	      ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   let separator = GMisc.separator `HORIZONTAL 
	       ~packing: ((box)#pack ~from:`END) () in
	   for i=1 to 1000000 do()done ;
	   if (box <> (List.hd (!branche_visitee))) then  
	     branche_visitee := [box]@(!branche_visitee);
	   button0#set_active true
	  ) 
	)with RegleNonApplicable -> (
	  erreur#set_text "RegleNonApplicable" ;
	  button0#set_active true
	 )
       | Erreur_syntaxe -> (
	   erreur#set_text "Erreur Syntaxique" ;
	   entry0#set_editable true;
	   entry1#set_editable true;
	   button0#set_active true
	  )
      )
				     ) ;
  
(* la fonction concernant le bouton de la rgle SND *) 
  button6#connect#toggled  ~callback:(fun _->
    if (button6#active)then
      (
       try(
	 if (entry1#editable ) then (
	   let a0 = entry0#text in
	   let a1 = entry1#text in
	   entry0#set_editable false;
	   entry1#set_editable false;
           if (a1 <> "  ")then (
	     let f0 = Stream.of_string a0 in
	     let f = Stream.of_string a1 in 
	     let l0 = analyseur_lexical2 f0 in
	     let l = analyseur_lexical2 f in
	     let t1 = (termes l )in
	     let e = env l0 in
	     let s1 = { env = e ; 
			lambda_terme = t1 ; 
			typage = SINGLE(generer()) 
		      } in
	     affiche_type#set_text (string_of s1.typage);	
	     let s2 = List.hd ( regle_snd s1 tab_types) in 
	     aff_cnt (!tab_types) box3;
	     let t = trans_to_string2 s2.env in
	     let txt =(t^" |- "^(trans_to_string s2.lambda_terme)
		       ^ " : "^(string_of(s2.typage)))  in
	     let box = List.hd (!branche_active) in
	     erreur#set_text "  ";
	     (GMisc.label  ~text:"(SND)" ~xalign:0.0 ~yalign:0.0 
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt 
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box)#pack ~from:`END) () in
	     list_sequent := [s2];
	     dern_regle := ["Snd"] ;
	     for i=1 to 1000000 do()done ;
             button0#set_active true
	    ) else (
	     entry0#set_editable true;
	     entry1#set_editable true;
	     erreur#set_text "entrez un lambda terme";
	     button0#set_active true;
	    )
	  )else(
	   let s1 = List.hd !list_sequent in
	   let s2 = List.hd ( regle_snd s1 tab_types) in 
	   aff_cnt (!tab_types) box3;	
	   let t = trans_to_string2 s2.env in
	   sequent_visite := [List.hd (!list_sequent)]@(!sequent_visite);
	   list_sequent := List.tl (!list_sequent) ;
	   list_sequent := [(s2)]@(!list_sequent) ;
	   dern_regle := ["Snd"]@(!dern_regle);
	   let txt =(t^" |- "^(trans_to_string s2.lambda_terme)
		     ^ " : "^(string_of(s2.typage)))  in
	   let box = List.hd (!branche_active) in
	   erreur#set_text "  ";
	   (GMisc.label  ~text:"(SND)" ~xalign:0.0 ~yalign:0.0 
	      ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label ~text:txt 
	      ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   let separator = GMisc.separator `HORIZONTAL 
	       ~packing: ((box)#pack ~from:`END) () in
	   for i=1 to 1000000 do()done ;
	   if (box <> (List.hd (!branche_visitee))) then  
	     branche_visitee := [box]@(!branche_visitee);
	   button0#set_active true
	  ) 
	)with RegleNonApplicable -> (
	  erreur#set_text "RegleNonApplicable" ;
	  button0#set_active true
	 )
       | Erreur_syntaxe -> (
	   erreur#set_text "Erreur Syntaxique" ;
	   entry0#set_editable true;
	   entry1#set_editable true;
	   button0#set_active true
	  )
      )
				     ) ;
 
(* la fonction concernant le bouton de la rgle INJL *)  
  button7#connect#toggled  ~callback:(fun _->
    if (button7#active)then
      (
       try(
	 if (entry1#editable ) then (
	   let a0 = entry0#text in
	   let a1 = entry1#text in
	   entry0#set_editable false;
	   entry1#set_editable false;
           if (a1 <> "  ")then (
	     let f0 = Stream.of_string a0 in
	     let f = Stream.of_string a1 in 
	     let l0 = analyseur_lexical2 f0 in
	     let l = analyseur_lexical2 f in
	     let t1 = (termes l )in 
	     let e = env l0 in
	     let s1 = { env = e ; 
			lambda_terme = t1 ; 
			typage = SINGLE(generer()) 
		      } in
	     affiche_type#set_text (string_of s1.typage);	
	     let s2 = List.hd ( regle_injl s1 tab_types) in 
	     aff_cnt (!tab_types) box3;
	     let t = trans_to_string2 s2.env in
	     let txt =(t^" |- "^(trans_to_string s2.lambda_terme)
		       ^ " : "^(string_of(s2.typage)))  in
	     let box = List.hd (!branche_active) in
	     erreur#set_text "  ";
	     (GMisc.label  ~text:"(INJL)" ~xalign:0.0 ~yalign:0.0 
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt 
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box)#pack ~from:`END) () in
	     list_sequent := [s2];
	     dern_regle := ["Injl"] ;
	     for i=1 to 1000000 do()done ;
             button0#set_active true
	    ) else (
	     entry0#set_editable true;
	     entry1#set_editable true;
	     erreur#set_text "entrez un lambda terme";
	     button0#set_active true;
	    )
	  )else(
	   let s1 = List.hd !list_sequent in
	   let s2 = List.hd ( regle_injl s1 tab_types) in 
	   aff_cnt (!tab_types) box3;
	   let t = trans_to_string2 s2.env in
	   sequent_visite := [List.hd (!list_sequent)]@(!sequent_visite);
	   list_sequent := List.tl (!list_sequent) ;
	   list_sequent := [s2]@(!list_sequent) ;
	   dern_regle := ["Injl"]@(!dern_regle);
	   let txt =(t^" |- "^(trans_to_string s2.lambda_terme)
		     ^ " : "^(string_of(s2.typage)))  in
	   let box = List.hd (!branche_active) in
	   erreur#set_text "  ";
	   (GMisc.label  ~text:"(INJL)" ~xalign:0.0 ~yalign:0.0 
	      ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label ~text:txt 
	      ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   let separator = GMisc.separator `HORIZONTAL 
	       ~packing: ((box)#pack ~from:`END) () in
	   for i=1 to 1000000 do()done ;
	   if (box <> (List.hd (!branche_visitee))) then  
	     branche_visitee := [box]@(!branche_visitee);
	   button0#set_active true
	  ) 
	)with RegleNonApplicable -> (
	  erreur#set_text "RegleNonApplicable" ;
	  button0#set_active true
	 )
       | Erreur_syntaxe -> (
	   erreur#set_text "Erreur Syntaxique" ;
	   entry0#set_editable true;
	   entry1#set_editable true;
	   button0#set_active true
	  )
      )
				     ) ;
  
(* la fonction concernant le bouton de la rgle INJR *) 
  button8#connect#toggled  ~callback:(fun _->
    if (button8#active)then
      (
       try(
	 if (entry1#editable ) then (
	   let a0 = entry0#text in
	   let a1 = entry1#text in
	   entry0#set_editable false;
	   entry1#set_editable false;
           if (a1 <> "  ")then (
	     let f0 = Stream.of_string a0 in
	     let f = Stream.of_string a1 in 
	     let l0 = analyseur_lexical2 f0 in
	     let l = analyseur_lexical2 f in
	     let t1 = (termes l )in 
	     let e = env l0 in
	     let s1 = { env = e ; 
			lambda_terme = t1 ; 
			typage = SINGLE(generer()) 
		      } in
	     affiche_type#set_text (string_of s1.typage);
	     let s2 = List.hd ( regle_injr s1 tab_types) in
	     aff_cnt (!tab_types) box3;
	     let t = trans_to_string2 s2.env in
	     let txt =(t^" |- "^(trans_to_string s2.lambda_terme)
		       ^ " : "^(string_of(s2.typage)))  in
	     let box = List.hd (!branche_active) in
	     erreur#set_text "  ";
	     (GMisc.label  ~text:"(INJR)" ~xalign:0.0 ~yalign:0.0 
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt 
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box)#pack ~from:`END) () in
	     list_sequent := [s2];
	     dern_regle := ["Injr"] ;
	     for i=1 to 1000000 do()done ;
             button0#set_active true
	    ) else (
	     entry0#set_editable true;
	     entry1#set_editable true;
	     erreur#set_text "entrez un lambda terme";
	     button0#set_active true;
	    )
	  )else(
	   let s1 = List.hd !list_sequent in
	   let s2 = List.hd ( regle_injr s1 tab_types) in 
	   aff_cnt (!tab_types) box3;
	   let t = trans_to_string2 s2.env in
	   sequent_visite := [List.hd (!list_sequent)]@(!sequent_visite);
	   list_sequent := List.tl (!list_sequent) ;
	   list_sequent := [s2]@(!list_sequent) ;
	   dern_regle := ["Injr"]@(!dern_regle);
	   let txt =(t^" |- "^(trans_to_string s2.lambda_terme)
		     ^ " : "^(string_of(s2.typage)))  in
	   let box = List.hd (!branche_active) in
	   erreur#set_text "  ";
	   (GMisc.label  ~text:"(INJR)" ~xalign:0.0 ~yalign:0.0 
	      ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label ~text:txt 
	      ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   let separator = GMisc.separator `HORIZONTAL 
	       ~packing: ((box)#pack ~from:`END) () in
	   for i=1 to 1000000 do()done ;
	   if (box <> (List.hd (!branche_visitee))) then  
	     branche_visitee := [box]@(!branche_visitee);
	   button0#set_active true
	  ) 
	)with RegleNonApplicable -> (
	  erreur#set_text "RegleNonApplicable" ;
	  button0#set_active true
	 )
       | Erreur_syntaxe -> (
	   erreur#set_text "Erreur Syntaxique" ;
	   entry0#set_editable true;
	   entry1#set_editable true;
	   button0#set_active true
	  )
      )
				     ) ;
  
(* la fonction concernant le bouton de la rgle CASE *) 
  button9#connect#toggled  ~callback:(fun _->
    if (button9#active)then
      (
       try(
	 if (entry1#editable ) then (
	   let a0 = entry0#text in
	   let a1 = entry1#text in
	   entry0#set_editable false;
	   entry1#set_editable false;
           if (a1 <> "  ")then (
	     let f0 = Stream.of_string a0 in
	     let f = Stream.of_string a1 in 
	     let l0 = analyseur_lexical2 f0 in
	     let l = analyseur_lexical2 f in
	     let t = (termes l )in
	     let e = env l0 in
	     let s0  = { env = e ; 
			 lambda_terme = t ; 
			 typage = SINGLE(generer()) 
		       } in
	     affiche_type#set_text (string_of s0.typage);
	     let s = regle_case s0 tab_types in 
	     aff_cnt (!tab_types) box3;
	     let s1 = List.hd s in 
	     let s2 = List.nth s 1 in
	     let s3 = List.nth s 2 in
	     let t1 = trans_to_string2 s1.env in
	     let t2 = trans_to_string2 s2.env in
	     let t3 = trans_to_string2 s3.env in
	     let txt1 =(t1^" |- "^(trans_to_string s1.lambda_terme)
			^ " : "^(string_of(s1.typage)))  in
	     let txt2 =(t2^" |- "^(trans_to_string s2.lambda_terme)
			^ " : "^(string_of(s2.typage)))  in
	     let txt3 =(t3^" |- "^(trans_to_string s3.lambda_terme)
			^ " : "^(string_of(s3.typage)))  in
	     let box = List.hd (!branche_active) in
	     erreur#set_text "  ";
	     let box02 =GPack.hbox ~homogeneous:true ~spacing:7
		 ~packing:((box)#pack ~from:`END) () in
	     let box03 = GPack.vbox ~packing:box02#pack () in
	     let box04 = GPack.vbox ~packing:box02#pack () in
	     let box05 = GPack.vbox ~packing:box02#pack () in
	     (GMisc.label  ~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~text:"(CASE)" ~xalign:0.0 ~yalign:0.0 
		~packing:((box03)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~packing:((box04)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~packing:((box05)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt1 ~packing:((box03)#pack ~from:`END)  
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt2 ~packing:((box04)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt3 ~packing:((box05)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box03)#pack ~from:`END) () in
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box04)#pack ~from:`END) () in
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box05)#pack ~from:`END) () in
	     list_sequent := [s1;s2;s3]@(!list_sequent);
	     dern_regle := ["Case"];
	     for i=1 to 1000000 do()done ;
	     if (box <> (List.hd (!branche_visitee))) then  
	       branche_visitee := [box]@(!branche_visitee);
	     branche_active := [box03]@[box04]@[box05];
             button0#set_active true
	    ) else (
	     entry0#set_editable true;
	     entry1#set_editable true;
	     erreur#set_text "entrez un lambda terme";
	     button0#set_active true;
	    )
	  )else(
	   let s0 = List.hd !list_sequent in
	   let s =  regle_case s0 tab_types in
	   aff_cnt (!tab_types) box3;
	   let s1 = List.hd s in
	   let s2 = List.nth s 1 in
	   let s3 = List.nth s 2 in
	   let t1 = trans_to_string2 s1.env in
	   let t2 = trans_to_string2 s2.env in
	   let t3 = trans_to_string2 s3.env in
	   let txt1 =(t1^" |- "^(trans_to_string s1.lambda_terme)
		      ^ " : "^(string_of(s1.typage)))  in
	   let txt2 =(t2^" |- "^(trans_to_string s2.lambda_terme)
		      ^ " : "^(string_of(s2.typage)))  in
	   let txt3 =(t3^" |- "^(trans_to_string s3.lambda_terme)
		      ^ " : "^(string_of(s3.typage)))  in
	   let box = List.hd (!branche_active) in
	   erreur#set_text "  ";
	   let box02 =GPack.hbox ~homogeneous:true ~spacing:7
	       ~packing:((box)#pack ~from:`END) () in
	   let box03 = GPack.vbox ~packing:box02#pack () in
	   let box04 = GPack.vbox ~packing:box02#pack () in
	   let box05 = GPack.vbox ~packing:box02#pack () in
	   (GMisc.label  ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label  ~text:"(CASE)" ~xalign:0.0 ~yalign:0.0 
	      ~packing:((box03)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label  ~packing:((box04)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label  ~packing:((box05)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label ~text:txt1 ~packing:((box03)#pack ~from:`END)  
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label ~text:txt2 ~packing:((box04)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label ~text:txt3 ~packing:((box05)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label  ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   let separator = GMisc.separator `HORIZONTAL 
	       ~packing: ((box03)#pack ~from:`END) () in
	   let separator = GMisc.separator `HORIZONTAL 
	       ~packing: ((box04)#pack ~from:`END) () in
	   let separator = GMisc.separator `HORIZONTAL 
	       ~packing: ((box05)#pack ~from:`END) () in
	   sequent_visite := [List.hd (!list_sequent)]@(!sequent_visite);
	   list_sequent := List.tl (!list_sequent);
	   list_sequent := [s1;s2;s3]@(!list_sequent);
	   dern_regle := ["Case"]@(!dern_regle);
	   for i=1 to 1000000 do()done ;
	   if (box <> (List.hd (!branche_visitee))) then  
	     branche_visitee := [box]@(!branche_visitee);
	   branche_active := List.tl (!branche_active);
	   branche_active :=  box03::box04::box05::(!branche_active);
           button0#set_active true;
	  ) 
	)with RegleNonApplicable -> (
	  erreur#set_text "RegleNonApplicable" ;
	  button0#set_active true
	 )
       | Erreur_syntaxe -> (
	   erreur#set_text "Erreur Syntaxique" ;
	   entry0#set_editable true;
	   entry1#set_editable true;
	   button0#set_active true
	  )
      )
				     ) ;
  
(* le bouton Recommencer *)
  b0#connect#clicked ~callback:(fun _->
    let l = box1#children in
    let l2 = box2#children in
    for i=0 to ((List.length (l))-3) do 
      (
       let n = (List.nth l i) in
       box1#remove (n);
      )done ;
    for i=0 to ((List.length (l2))-3) do 
      (
       let n = (List.nth l2 i) in
       box2#remove (n);
      )done ;
    for i=0 to ((List.length (!branche_visitee))-1) do 
      (
       let n = (List.nth (!branche_visitee) i) in
       if (n <> box1) then vider n;
      )done ;
    for i=0 to ((List.length (!branche_active))-1) do 
      (
       let n = (List.nth (!branche_active) i) in
       if (n <> box1) then vider n;
      )done ;
    vider box3;
    branche_active := [box1];
    branche_visitee:= [box1];
    list_sequent := [] ;
    sequent_visite := [];
    dern_regle := [];
    tab_types := [];
    i := 0;
    entry0#set_editable true;
    entry1#set_editable true;
    entry0#set_text "  ";
    entry1#set_text "  ";
    erreur#set_text "  ";
    affiche_type#set_text "  ";
    button0#set_active true
			       );	    
  
  window#show ();
  Main.main ()
    
let _ = main ()
    
